# Generated by Django 4.1.7 on 2023-03-25 23:03

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('outbox_hitcount', '0009_remove_hitlocation_name_hitlocation_city_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='hitcount',
            name='site',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site'),
        ),
    ]



import datetime

import pytz
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.cache import cache
from django.shortcuts import render
from django.utils import timezone
from hitcount.models import Hit, HitCount

from .common_date import add_months, get_week_date


def get_statistic(site_id, is_cache=False): 
    ''' 
        Tambah fitur cache, jika True maka cache data statistik sesuai dengan data yg tidak berubah lagi untuk 1 hari
        1 Jam = 60 Menit = 3600 Detik
        24 Jam = 24 * 3600 Detik = 86400 Detik
    '''
    time_zone = getattr(settings, 'TIME_ZONE', 'UTC') # get setting timezone
    tz = pytz.timezone(time_zone)   
    context = {}
    # tgl = datetime.datetime.now()
    tgl = timezone.now()
    # Domain = request.get_host()    
    # site_id = get_site_id(request)

    # Update 1 Des 2022
    # Tambah hitcount_id dari tabel Hit agar perhitungan khusus model tertentu saja
    # tahapan untuk mendapatkan hitcount_id adalah :
    # 1. Aktifkan model (get data model)
    # 2. Dapatkan content_type_id
    # 3. Dapatkan hitcount ID
    
    # implementasi :
    # 1. site_id sudah ada (site_id variabel di atas)
    # 2. content_type_id
    content_type_id = ContentType.objects.get(app_label='sites', model='site')
    content_type_id = content_type_id.id if content_type_id else None   # harus ada None
    
    ## CARA LAIN DAPATKAN CONTENT TYPE
    # content_type = ContentType.objects.filter(model='site').first()
    
    # 3. Dapatkan hitcount ID
    hitcount_id = HitCount.objects.filter(content_type_id=content_type_id, object_pk=site_id).first()
    hitcount_id = hitcount_id.id if hitcount_id else None

    # hitcount_id di gunakan di semua kondisi di bawah ini ....
    # kondisi DOMAIN sudah tidak diperlukan lagi
    # Hal ini membuktikan tambahan field Domain di model hit dan hitcount sebenarnya tidak di perlukan 
    # (Update langsung dari hitcount source menjadi mungkin dilakukan)
    # ---------------------------------------------------------

    # start date jika mengabaikan jam
    # maka bagian jam 0:0:0 mulai pukul 0 (benar)
    # Jika mengabaikan jam, seharusnya end data berakhiran 23:59:59 bukan 0:0:0
    # supaya end data tetap dihitung
    # end_date = datetime.date(tgl.year, tgl.month, tgl.day)
    
    # RULE CAHCE :
    # TIDAK DI CACHE (Mulai di CACHE yesterday dst)
    # TIDAK DI CACHE online juga
    # timeout = 86400 # in second = 1 Day
    # Perubahan setelah jam 00:00:00 malam atau setelah pergantian hari
    # buat cache ini expired saat pergantian hari
    # hitung jumlah detik dari waktu sekarang sampai jam 00:00:00 nanti malam
    # Plus 1 menit untuk antisipasi expired sebelum jam 00:00
    # jika expired sebelum jam 00:00 kemidan hit count di update ke cache maka satu hari full data akan salah karena masih mengambil data kemarin
    # tgl = waktu sekarang
    tgl00 = tgl + datetime.timedelta(days=1)
    jam00 = datetime.datetime(tgl00.year, tgl00.month, tgl00.day, 0, 1, 0, tzinfo=tz) # 60 adalah tambahan waktu 1 menit untuk antisipasi kesalahan refresh data ke cache
    # jam00 = tz.localize(jam00)
    timeout = (jam00-tgl00).seconds
    selisih = 0 # untuk pengujung tambahan yg terhitung mulai jam 00:00

    # PERUBAHAN RULE, khusus unutk minggu ini dan bulan ini
    # Gunakan cara, update manual tanpa membaca database lagi
    # Misal data pengunjung hari ini sudah disimpan 5
    # kemudian 1 jam berikutnya menjadi 8
    # berarti ada penambahan 3, maka
    # tambah juga data 3 ini ke pengunjung minggu ini dan bulan ini
    # HIT_TODAY digunakan untuk menyimpan pengunjung update jam 00:00
    # cari selisih dnegan pengunjung hari ini, hasilnya
    # tambahkan ke pengujung minggu ini dan bulan ini


    # a, b = divmod(timeout, 3600)
    # print('expired in next', str(a), str(b//60))
    tmp = 'hit_today'
    tmp_cache = cache.get(tmp, version=site_id)

    if not (is_cache and tmp_cache is not None): # karena kondisi tmp_cache = 0 maka tetap masuk kondisi load from DB
        print('load from DB (' + tmp + ')')

        # Abaikan jam, ambil hari ini saja
        # hit_today = Hit.objects.filter(hitcount_id=hitcount_id, created__year=tgl.year, created__month=tgl.month, created__day=tgl.day)
        hit_today = Hit.objects.filter(hitcount_id=hitcount_id, created__date=tgl.date())
        # print('hit_today',hit_today)

        tmp_cache = hit_today.count() if hit_today else 1
        
        # save to cache for next time
        cache.set(tmp, tmp_cache, timeout, version=site_id)

        # return to context (Ini di update jam 00:00)
        context[tmp] = tmp_cache
    else: 
        # Ini di UPDATE tiap pengujung datang

        # Abaikan jam, ambil hari ini saja
        # hit_today = Hit.objects.filter(hitcount_id=hitcount_id, created__year=tgl.year, created__month=tgl.month, created__day=tgl.day)
        hit_today = Hit.objects.filter(hitcount_id=hitcount_id, created__date=tgl.date)
        # print('hit_today',hit_today)
        context[tmp] = hit_today.count() if hit_today else 1

        selisih = context[tmp] - tmp_cache
        print('selisih = ', selisih)
    # if hit_today:        
    #     context['hit_today'] = hit_today.count()
    # else:
    #     context['hit_today'] = 1    # paling tidak ada 1 yg berkunjung hari ini (diri anda sendiri)

    # ref : https://docs.djangoproject.com/en/4.1/topics/cache/
    # 1. kemarin
    # ----------------------------------------------------------------------
    tmp = 'hit_yesterday'
    tmp_cache = cache.get(tmp, version=site_id)
    # tmp_cache = 0 if tmp_cache is None else tmp_cache
    
    # print('cache=',tmp_cache)

    if not (is_cache and tmp_cache is not None): # karena kondisi tmp_cache = 0 maka tetap masuk kondisi load from DB
        print('load from DB (' + tmp + ')')
        # context[tmp] = tmp_cache
        # else:    
        # abaikan jam juga, ambil hari kemarin
        start_date = tgl + datetime.timedelta(days=-1)
        # start_date = datetime.date(start_date.year, start_date.month, start_date.day)   # abaikan jam
        # end_date = datetime.date(tgl.year, tgl.month, tgl.day)
        tmp_cache = Hit.objects.filter(hitcount_id=hitcount_id, \
            created__year=start_date.year, created__month=start_date.month, created__day=start_date.day).count()
        # tmp_cache = Hit.objects.filter(hitcount_id=hitcount_id, \
        #     created__date=start_date.date).count()

        # save to cache for next time
        cache.set(tmp, tmp_cache, timeout, version=site_id)
    
    # Return to context        
    context[tmp] = tmp_cache

    # 2. Minggu INI (Tidak di cache, karena selalu berubah, jika pengunjung hari ini bertambah
    # Data minggu ini juga bertambah)
    # Rule berubah khusus minggu ini dan bulan ini
    # ----------------------------------------------------------------------
    tmp = 'hit_this_week'
    tmp_cache = cache.get(tmp, version=site_id)

    if not (is_cache and tmp_cache is not None): # karena kondisi tmp_cache = 0 maka tetap masuk kondisi load from DB
        print('load from DB (' + tmp + ')')
        # abaikan jam, ambil data dalam 1 minggu
        start_date, end_date = get_week_date(tgl.year, tgl.month, tgl.day)
        start_date = tz.localize(start_date)
        end_date = tz.localize(end_date)
        # print(start_date, end_date)

        # start_date = tgl + datetime.timedelta(days=-7)
        # start_date = datetime.date(start_date.year, start_date.month, start_date.day) # abaikan jam        
        # end_date = datetime.datetime(tgl.year, tgl.month, tgl.day, 23, 59, 59)

        tmp_cache = Hit.objects.filter(hitcount_id=hitcount_id, \
            created__range=(start_date, end_date)).count()
        # save to cache for next time
        cache.set(tmp, tmp_cache, timeout, version=site_id)

    tmp_cache += selisih # Tambahan ini saja
    context[tmp] = tmp_cache


    # 3. Minggu LALU
    # ----------------------------------------------------------------------
    tmp = 'hit_last_week'
    tmp_cache = cache.get(tmp, version=site_id)

    if not (is_cache and tmp_cache is not None): # karena kondisi tmp_cache = 0 maka tetap masuk kondisi load from DB
        print('load from DB (' + tmp + ')')
        # abaikan jam, ambil data 1 minggu yg lalu
        # start_date = tgl + datetime.timedelta(days=-14)
        # start_date = datetime.date(start_date.year, start_date.month, start_date.day) # abaikan jam
        # end_date = tgl + datetime.timedelta(days=-7)
        # end_date = datetime.datetime(end_date.year, end_date.month, end_date.day, 23, 59, 59) # abaikan jam
        start_date, end_date = get_week_date(tgl.year, tgl.month, tgl.day)
        start_date = start_date + datetime.timedelta(days=-7)
        end_date = end_date + datetime.timedelta(days=-7)
        start_date = tz.localize(start_date)
        end_date = tz.localize(end_date)
        
        # print('=', start_date, end_date)

        tmp_cache = Hit.objects.filter(hitcount_id=hitcount_id, \
            created__range=(start_date, end_date)).count()
        # save cache
        cache.set(tmp, tmp_cache, timeout, version=site_id)

    context[tmp] = tmp_cache


    # 4. BULAN INI
    # ----------------------------------------------------------------------
    tmp = 'hit_this_month'
    tmp_cache = cache.get(tmp, version=site_id)

    if not (is_cache and tmp_cache is not None): # karena kondisi tmp_cache = 0 maka tetap masuk kondisi load from DB
        print('load from DB (' + tmp + ')')

        # abaikan jam, ambil data bulan ini () (Bukan tgl sekarang sampai 1 bulan ke belakang, karena berbeda jumlah hari dalam 1 bulan)
        tmp_cache = Hit.objects.filter(hitcount_id=hitcount_id, \
            created__year=tgl.year, created__month=tgl.month).count()

        # save cache
        cache.set(tmp, tmp_cache, timeout, version=site_id)

    tmp_cache += selisih
    context[tmp] = tmp_cache


    # 5. BULAN LALU
    # ----------------------------------------------------------------------
    tmp = 'hit_last_month'
    tmp_cache = cache.get(tmp, version=site_id)

    if not (is_cache and tmp_cache is not None): # karena kondisi tmp_cache = 0 maka tetap masuk kondisi load from DB
        print('load from DB (' + tmp + ')')

        # abaikan jam, ambil dari bulan lalu
        start_date = add_months(tgl,-1)
        tmp_cache = Hit.objects.filter(hitcount_id=hitcount_id, \
            created__year=start_date.year, created__month=start_date.month).count()
        # save cache
        cache.set(tmp, tmp_cache, timeout, version=site_id)

    context[tmp] = tmp_cache

    # abikan menit dan detik
    # ambil 3 jam terakhir untuk penanda user online (ubah dari 1 jam menjadi 3 jam (agar tampak lebih banyak))
    

    # Update 1 Des 2022, 
    # Ambil 5 jam terakhir dan ip unique
    start_date = tgl + datetime.timedelta(hours=-5) # abaikan menit dan detik
    start_date = datetime.datetime(start_date.year, start_date.month, start_date.day, start_date.hour, 0, 0, tzinfo=tz) # abaikan menit dan detik
    # end_date = tgl
    end_date = datetime.datetime(tgl.year, tgl.month, tgl.day, tgl.hour, 59, 59, tzinfo=tz) # abaikan menit dan detik
    hit_online = Hit.objects.filter(hitcount_id=hitcount_id, \
        created__range=(start_date, end_date)).values('user_agent').order_by('user_agent').distinct()

    # IP masih sama, ganti menjadi user_agent

    # Ambil data 3 jam terakhir, dengan ip unique, itu dianggap jumlah user yg sedang online
    context['hit_online'] = hit_online.count() if hit_online else 1

    # if hit_online:        
    #     context['hit_online'] = hit_online.count() 
    # else: # tetap perlu 1, karena pengunjung yg sama tidak di hitung lagi sampai hari berikutnya
    #     context['hit_online'] = 1 # paling tidak ada 1 yg online

    # context['hit_all'] = Hit.objects.filter(domain=Domain).count()
    # hit terbaru di ambil dari ringkasan, karena mencakup seluruh pengunjung dari awal website online
    # data ada di hit_count bukan di hit (karena tabel ini akan di hapus dalam waktu 2 bulan, secara manual di hapus)

    # Update di OPD (pending)

    ## CARA LAIN DAPATKAN CONTENT TYPE
    # content_type = ContentType.objects.filter(model='site').first()
    tmp = 'hit_all'
    tmp_cache = cache.get(tmp, version=site_id)

    if not (is_cache and tmp_cache is not None): # karena kondisi tmp_cache = 0 maka tetap masuk kondisi load from DB
        print('load from DB (' + tmp + ')')


        hit_count = HitCount.objects.filter(object_pk = site_id, content_type_id = content_type_id) #, domain=Domain)
        tmp_cache = hit_count[0].hits if hit_count else 1
        # if hit_count:
        #     context['hit_all'] = hit_count[0].hits
        # else:   
        #     context['hit_all'] = 1 # mengikuti jumlah site Online dan hitcount 1 hari 
        cache.set(tmp, tmp_cache, timeout, version=site_id)

    tmp_cache += selisih
    context[tmp] = tmp_cache

    # RETURN
    return context



# Generated by Django 4.1.7 on 2023-03-21 04:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='HitBrowser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='HitDevice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='HitLocation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('ip_address', models.CharField(max_length=20)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='HitOS',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='HitCount',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('count', models.PositiveBigIntegerField(default=0)),
                ('end_date', models.DateTimeField(auto_now=True)),
                ('browser_count', models.PositiveBigIntegerField(default=0)),
                ('os_count', models.PositiveBigIntegerField(default=0)),
                ('device_count', models.PositiveBigIntegerField(default=0)),
                ('location_count', models.PositiveBigIntegerField(default=0)),
                ('object_pk', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
                ('hits_browser', models.ManyToManyField(to='outbox_hitcount.hitbrowser')),
                ('hits_device', models.ManyToManyField(to='outbox_hitcount.hitdevice')),
                ('hits_location', models.ManyToManyField(to='outbox_hitcount.hitlocation')),
                ('hits_os', models.ManyToManyField(to='outbox_hitcount.hitos')),
            ],
        ),
    ]



from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site

class HitLocation(models.Model):
    '''
        Table Master
        Cek location unique, do not duplicate value in name
        192.168.13.29
    '''
    country = models.CharField(max_length=50, default=None, null=True)
    city = models.CharField(max_length=50, default=None, null=True)
    ip_address = models.CharField(max_length=20)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

class HitDevice(models.Model):
    '''
        Table Master
        Cek device unique, do not duplicate value in name
        example:
        Tab, Hp, Desktop/Laptop
    '''
    name = models.CharField(max_length=50)
    version = models.CharField(max_length=20)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

class HitOS(models.Model):
    '''
        Table Master
        Cek Operating System unique, do not duplicate value in name
        Linux, mac os, Windows, android
    '''
    name = models.CharField(max_length=50)
    version = models.CharField(max_length=20)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

class HitBrowser(models.Model):
    '''
        Table Master
        Cek Browser unique, do not duplicate value in name
        Firefox, chrome, 
    '''
    name = models.CharField(max_length=50)
    version = models.CharField(max_length=20)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

class HitCount(models.Model):
    """
    Table Transaksi (jumlah hit)

    Base class for hitcount models.

    Model that stores the hit totals for any content object.

    """
    # count = models.PositiveIntegerField(default=0)
    count = models.PositiveBigIntegerField(default=0)
    end_date = models.DateTimeField()  # last date in month (ringkasan dari tanggal 1, simpan sebagai tanggal akhir dari tiap bulan)
    site = models.ForeignKey(Site, on_delete=models.CASCADE, null=True, blank=True)

    # count di atas, misal berjumlah 10
    # maka hit browser misal chrome, firefox, dll, jumlahnya harus 10
    # hit os, misal andorid, linux, windows, jumlahnya harus 10 juga

    hits_browser = models.ManyToManyField(HitBrowser) #, through="HitCountBrowser")
    # field ini pindahan dari tabel intemediate, lebih tepat di letakkan di tabel ini
    # browser_count = models.PositiveBigIntegerField(default=0)    # ukuran gambar width

    hits_os = models.ManyToManyField(HitOS) #, through="HitCountOS")
    # os_count = models.PositiveBigIntegerField(default=0)    # ukuran gambar width

    hits_device = models.ManyToManyField(HitDevice) #, through="HitCountDevice")
    # device_count = models.PositiveBigIntegerField(default=0)    # ukuran gambar width

    hits_location = models.ManyToManyField(HitLocation) #, through="HitCountLocation")
    # location_count = models.PositiveBigIntegerField(default=0)    # ukuran gambar width

    # link ke object, bisa berita, sita, artikel, dll
    # , related_name="content_type_set_for_%(class)s"
    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE)
    object_pk = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_pk')


# class HitCountBrowser(models.Model):
#     '''
#         # Tabel pertengahan di relasi many2many dengan tambahan field
#         Many to many relationship with additional fields
#     '''
#     hit_count = models.ForeignKey(HitCount, on_delete=models.CASCADE)
#     hit_browser = models.ForeignKey(HitBrowser, on_delete=models.CASCADE)

#     # count = models.PositiveBigIntegerField(default=0)    # ukuran gambar width
#     # Fitur cropping mengikuti data ini
#     # image_width = models.SmallIntegerField()    # ukuran gambar width
#     # image_height = models.SmallIntegerField()   # ukuran gambar height

# class HitCountOS(models.Model):
#     '''
#         # Tabel pertengahan di relasi many2many dengan tambahan field
#         Many to many relationship with additional fields
#     '''
#     hit_count = models.ForeignKey(HitCount, on_delete=models.CASCADE)
#     hit_os = models.ForeignKey(HitOS, on_delete=models.CASCADE)

#     # count = models.PositiveBigIntegerField(default=0)    # ukuran gambar width

# class HitCountDevice(models.Model):
#     '''
#         # Tabel pertengahan di relasi many2many dengan tambahan field
#         Many to many relationship with additional fields
#     '''
#     hit_count = models.ForeignKey(HitCount, on_delete=models.CASCADE)
#     hit_device = models.ForeignKey(HitDevice, on_delete=models.CASCADE)

#     # count = models.PositiveBigIntegerField(default=0)    # ukuran gambar width

# class HitCountLocation(models.Model):
#     '''
#         # Tabel pertengahan di relasi many2many dengan tambahan field
#         Many to many relationship with additional fields
#     '''
#     hit_count = models.ForeignKey(HitCount, on_delete=models.CASCADE)
#     hit_location = models.ForeignKey(HitLocation, on_delete=models.CASCADE)

#     # count = models.PositiveBigIntegerField(default=0)    # ukuran gambar width



# Generated by Django 4.1.7 on 2023-03-25 22:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outbox_hitcount', '0008_remove_hitcount_browser_count_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='hitlocation',
            name='name',
        ),
        migrations.AddField(
            model_name='hitlocation',
            name='city',
            field=models.CharField(default=None, max_length=50),
        ),
        migrations.AddField(
            model_name='hitlocation',
            name='country',
            field=models.CharField(default=None, max_length=50),
        ),
    ]



"""
Provides outbox_hitcount version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update outbox_hitcount` to change this file.

from incremental import Version

__version__ = Version("outbox_hitcount", 1, 0, 10)
__all__ = ["__version__"]



# Generated by Django 4.1.7 on 2023-03-25 00:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outbox_hitcount', '0007_alter_hitbrowser_version_alter_hitdevice_version_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='hitcount',
            name='browser_count',
        ),
        migrations.RemoveField(
            model_name='hitcount',
            name='device_count',
        ),
        migrations.RemoveField(
            model_name='hitcount',
            name='location_count',
        ),
        migrations.RemoveField(
            model_name='hitcount',
            name='os_count',
        ),
        migrations.AlterField(
            model_name='hitbrowser',
            name='version',
            field=models.CharField(max_length=20),
        ),
        migrations.AlterField(
            model_name='hitdevice',
            name='version',
            field=models.CharField(max_length=20),
        ),
        migrations.AlterField(
            model_name='hitos',
            name='version',
            field=models.CharField(max_length=20),
        ),
    ]



import calendar
import datetime
import json
import os
import random
import time
from datetime import timedelta

import httpagentparser
import pytz
import requests
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.exceptions import FieldDoesNotExist
from django.db import transaction
from django.db.models import F, Q
from django.utils import timezone
from hitcount.models import Hit

from outbox_hitcount.models import *

from .common_date import add_months, get_last_day_of_month

# from urllib2 import urlopen

def get_geolocation_opt3(str_ip_address):
    '''
        Save IP address first if request json geolocation not exists
    '''
    res = requests.get('http://ip-api.com/json/' + str_ip_address) # 45 request per minute
    if res:
        res = res.json()
        if res['status'] =='success':
            return (res['country'], res['city'])
    return None

def get_geolocation_opt1(str_ip_address):
    '''
        Save IP address first if request json geolocation not exists
        exp: 180.243.14.149
    '''
    result = os.popen("curl https://ipapi.co/"+ str_ip_address +"/json/").read() # 1000 per hari
    if result:
        tmp = json.loads(result)
        return (tmp['country_name'], tmp['city'])
    return None    

def get_geolocation_opt2(str_ip_address):
    '''
        Save IP address first if request json geolocation not exists
    '''        
    res = requests.get('http://ipwho.is/' + str_ip_address) # 10.000 request per minute    
    if res:
        res = res.json()
        if res['success']:
            return (res['country'], res['city'])                    
    return None 

def get_geolocation_opt4(str_ip_address):
    '''
        Save IP address first if request json geolocation not exists
    '''        
    result = os.popen("curl http://api.db-ip.com/v2/free/"+ str_ip_address).read() # 1000 per hari
    if result:
        tmp = json.loads(result)
        return (tmp['countryName'], tmp['city'])
    return None     

# ada version yg belum bagus di split sehingga tersimpan 
# X 10.15.5) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.1.1 Safari/605.1.15 (Applebot/0.1
def correct_version(version):
    if version.strip():
        tmp = version.replace("(","<-!->")
        tmp = tmp.replace(")","<-!->")
        return tmp.split("<-!->")[0]
    return version

def is_field_exists(model, field):
    # for field in cls._meta.get_fields(include_hidden=True):
    #     if field.name == field:
    #         return True
    # return False
    if model:
        try:
            # field = model._meta.get_field(field)
            for i in model._meta.get_fields():
                if (not i.many_to_many) and (i.name==field):
                    return True

        except FieldDoesNotExist:
            return False

        # return False    # tidak ada error, tapi data tidak ditemukan
    return False

# def get_for_object(obj, end_date):
#     ctype = ContentType.objects.get_for_model(obj)    
#     hit_count, created = get_or_create(content_type=ctype, object_pk=obj.pk)
#     return hit_count

def get_or_set_browser(browser):
    name = browser.get('name') if browser else None    
    version = browser.get('version') if browser else None
    
    # jika masih kosong karena tidak ada data maka set not defined
    if not name: name = 'none'
    if not version: version = 'none'
    version = correct_version(version)

    browser, created = HitBrowser.objects.get_or_create(name=name, version=version)
    return browser    

def get_or_set_os(os):
    name = os.get('name') if os else None    
    version = os.get('version') if os else None
    
    # jika masih kosong karena tidak ada data maka set not defined
    if not name: name = 'none'
    if not version: version = 'none'
    version = correct_version(version)

    os, created = HitOS.objects.get_or_create(name=name, version=version)
    return os

def get_or_set_device(device):
    name = device.get('name') if device else None    
    version = device.get('version') if device else None
    
    # jika masih kosong karena tidak ada data maka set not defined
    if not name: name = 'none'
    if not version: version = 'none'
    version = correct_version(version)

    device, created = HitDevice.objects.get_or_create(name=name, version=version)
    return device

def get_or_set_location(ip_address):
    '''
        Location: '192.168.13.29' format string IP ADDRESS
        Khusus location field version tidak ada di ganti ip_address
        kode unik cukup IP address saja
    '''
    # ip_address = location.get('ip_address') if location else None        
    
    # jika masih kosong karena tidak ada data maka set not defined
    if not ip_address: ip_address = 'none'    

    location, created = HitLocation.objects.get_or_create(ip_address=ip_address)
    return location

def hitcount_insert_m2m_field(hit_count, browser, os, platform, ip_address):    # param type dict
    # hit_count = data.get('hit_count')
    # browser = data.get('browser')
    # os = data.get('os')
    # platform = data.get('platform')
    # ip_address = data.get('ip_address')

    obj = get_or_set_browser(browser)
    hit_count.hits_browser.add(obj)
    # hit_count.browser_count += 1

    # get or set os
    obj = get_or_set_os(os)
    hit_count.hits_os.add(obj)
    # hit_count.os_count += 1

    # get or set device
    obj = get_or_set_device(platform)
    hit_count.hits_device.add(obj)
    # hit_count.device_count += 1

    # get or set location
    obj = get_or_set_location(ip_address)
    hit_count.hits_location.add(obj)
    # hit_count.location_count += 1

    # hit_count.save()

def special_condition(object_pk, end_date, data):
    '''
        Special condition:
        if site_id exists
            but object_pk not found, search on other model
            update hitcount for all model
    '''
    # jika di model yg aktif tidak ada
    model_priority = ['artikel', 'berita', 'galery_video', 'galery_foto', 'halaman_statis', 'pengumuman', 'social_media'] # 'link_terkait', 

    for i in model_priority:
        # print('proses', i)
        ct = ContentType.objects.filter(model=i)
        if ct:
            ct_class = ct.get().model_class()
            if is_field_exists(ct_class, 'site'):
                obj = ct_class.objects.filter(id=object_pk) # cari site_id dari model
                site_id = None
                if obj:                    
                    site_id = obj.get().site_id

                    if site_id:
                        site = Site.objects.filter(id=site_id) # cari nama site dari site_id yg di dapat
                        if site:
                            site = site.get()
                            content_type_site = ContentType.objects.get_for_model(site)

                            hit_count, created = HitCount.objects.get_or_create(
                                content_type=content_type_site, 
                                object_pk=site_id,
                                defaults={'end_date': end_date, 'site_id': site_id}
                            )
                            hit_count.count += 1
                            
                            # hit_count.update(count=F(count)+1)

                            data = {
                                'hit_count': hit_count,
                                'browser': data['browser'],
                                'os': data['os'],
                                'platform': data['platform'],
                                'ip_address': data['ip_address']
                            }
                            hitcount_insert_m2m_field(**data)
                            hit_count.save()        
                            print('proses', i, 'saved')
                    

@transaction.atomic
def do_summary(qs, end_date):
    j=0
    count = qs.count()
    # result = False
    try:
        for i in qs:
            ip_address = i.ip
            user_agent = i.user_agent
            user_agent_parser = httpagentparser.detect(user_agent)

            platform = user_agent_parser.get('platform')
            os = user_agent_parser.get('os')
            # bot = user_agent_parser.get('bot')
            # dist = user_agent_parser.get('dist')
            browser = user_agent_parser.get('browser')

            # hitcount_id = i.hitcount.id
            object_pk = i.hitcount.object_pk
            print('---')
            # print()

            content_type = i.hitcount.content_type
            content_type_id = content_type.id        
            j+=1
            print(int(j), 'of', int(count), 'object_pk', object_pk, 'model', content_type.model, 'hitcount_id', i.hitcount.id)
            site_id = None

            # dari content type ubah mejadi object
            # dari object, cek apakah ada field site_id
            # jika ada ambil PK dari object ini
            # ct = ContentType.objects.get_for_id(content_type_id)

            # print('content type=', content_type)

            ct_class = content_type.model_class()
            
            # print('ct_class=', ct_class)

            # print('ct=', ct)
            # ct_class = ct.model_class()
            # print('ct_class=', ct_class)

            # Jika ct_class tidak ada berarti model tersebut tidak di temukan di project
            # misal galery_video
            if ct_class:
                # print(ct_class._meta.get_fields())

                # obj = ct.get_object_for_this_type(id=object_pk)
                # print('ct_class=', ct_class)
                
                # print('obj=', obj)
                #             
                # cek apakah ada field site ID    
                mfound = False        
                if is_field_exists(ct_class, 'site'):
                    obj = ct_class.objects.filter(id=object_pk) # cari site_id dari model
                    if obj:                    
                        site_id = obj.get().site_id
                        print('site_id', site_id)
                        mfound = True
                    else:
                        print('site_id', object_pk, 'tidak ditemukan!')
                else:
                    print('site_id tidak ditemukan di model')

                if not mfound:
                    data = {
                        'hit_count': None,
                        'browser': browser,
                        'os': os,
                        'platform': platform,
                        'ip_address': ip_address
                    }

                    special_condition(object_pk, end_date, data)

            # 1. jika ada field site_id, maka insert summary baru content_type = site
            if site_id:
                site = Site.objects.filter(id=site_id) # cari nama site dari site_id yg di dapat
                if site:
                    site = site.get()
                    content_type_site = ContentType.objects.get_for_model(site)

                    hit_count, created = HitCount.objects.get_or_create(
                        content_type=content_type_site, 
                        object_pk=site_id,
                        defaults={'end_date': end_date, 'site_id': site_id}
                    )
                    hit_count.count += 1
                    
                    # hit_count.update(count=F(count)+1)

                    data = {
                        'hit_count': hit_count,
                        'browser': browser,
                        'os': os,
                        'platform': platform,
                        'ip_address': ip_address
                    }
                    hitcount_insert_m2m_field(**data)
                    hit_count.save()                

            # 2. default insert content_type dari apa adanya data di Hit
            # content_type = ContentType.objects.get_for_model(i)
            if site_id:
                hit_count, created = HitCount.objects.get_or_create(
                    content_type=content_type,  # data sudah ada di paling atas
                    object_pk=object_pk,
                    defaults={'end_date': end_date, 'site_id': site_id}
                )
            else:
                hit_count, created = HitCount.objects.get_or_create(
                    content_type=content_type,  # data sudah ada di paling atas
                    object_pk=object_pk,
                    defaults={'end_date': end_date, 'site_id': None}
                )

            hit_count.count += 1
            # hit_count.save()
            # hit_count.update(count=F(count)+1)

            data = {
                'hit_count': hit_count,
                'browser': browser,
                'os': os,
                'platform': platform,
                'ip_address': ip_address
            }
            hitcount_insert_m2m_field(**data)
            hit_count.save()

        clear_summary_qs(qs)
    except:
        print('something goes wrong!')
        return False
    
    return True

# @transaction.atomic
def clear_summary_qs(qs):
    '''
        Clear query set yg berhasil di execute
    '''    
    number_removed = qs.count()
    qs.delete()
    self.stdout.write('Successfully removed %s Hits' % number_removed)


def auto_hit_summary(month_count=-1): # proses jumlah bulan, jika -1 maka semua di proses
    '''
        Should be auto run in midnight
    '''
    time_zone = getattr(settings, 'TIME_ZONE', 'UTC') # get setting timezone
    tz = pytz.timezone(time_zone)    

    grace = getattr(settings, 'HITCOUNT_KEEP_HIT_IN_DATABASE', {'days': 30})
    period = timezone.now() - timedelta(**grace)
    # qs = Hit.objects.filter(created__lt=period)

    # seluruh data yg akan diringkas ada di qs
    # filter lagi per bulan
    # if qs:

    # ambil bulan dan tahun untuk di filter lagi
    # first_data = qs[0]

    # month_count = 1 # looping sejumlah month_count, jika -1 berarti semua data
    mcount = 5 # batasi looping 5 kali jika hasil query set kosong
    # month = period.month
    # year = period.year
    # end_day_of_month = get_last_day_of_month(year, month)    # return hari

    # # dapatkan range tanggal yang benar
    # begin_date = datetime.date(year, month, 1) # pukul 0:0:0
    # end_date = datetime.date(year, month, end_day_of_month, 23, 59, 59)
    tmp = add_months(period, 1) # karena proses awal add_month2 -1 maka di add dulu disini
    begin_date = datetime.datetime(tmp.year, tmp.month, 1)

    # mulai ambil data di database
    while mcount > 0:
        begin_date = add_months(begin_date, -1)
        year = begin_date.year
        month = begin_date.month
        end_day_of_month = get_last_day_of_month(year, month)    # return hari
        end_date = datetime.datetime(year, month, end_day_of_month, 23, 59, 59)
        
        # add time zone
        begin_date = tz.localize(begin_date)
        end_date = tz.localize(end_date)

        qs = Hit.objects.filter(created__gte=begin_date, created__lte=end_date)
        if not qs:
            mcount -= 1
        else:
            if do_summary(qs, end_date):
                # print('Begin clear summary')
                # clear_summary_qs(qs) # pindahkan di dalam modul do_summary
                print('Complete')
            else:
                return False    # jika do_summary gagal di eksekusi, maka keluar looping

            # else:
            #     print('Not Complete')
            
            if month_count > 0: month_count -= 1
            if month_count == 0: break
            
    return True

@transaction.atomic                            
def auto_get_location(request_per_minute=30, max_data=500):   
    '''
        Batasi hit per ment 30 saja agar tidak di banned oleh situs gratisan
        jalankan menggunakan celery
        batasi max data 500
    '''
    start_time = datetime.datetime.now()
    stop_time = start_time + timedelta(minutes=1)

    hit_location = HitLocation.objects.filter(Q(country='') | Q(city=''))[:max_data]
    count = 0
    waiting_list = [1, 2, 3, 4, 5, 6, 7]  # random list
    loc = 'loc1'

    for i in hit_location:
        count += 1
        ip_address = i.ip_address
        if count<=request_per_minute:
            location = get_geolocation_opt1(ip_address)
            loc = 'loc1'

            if not location: 
                location = get_geolocation_opt2(ip_address)
                loc = 'loc2'

            if not location: 
                location = get_geolocation_opt3(ip_address)
                loc = 'loc3'

            if not location: 
                location = get_geolocation_opt4(ip_address)
                loc = 'loc4'

            if not location:
                print('Location Not Found', ip_address)
                loc = 'none'
            else:
                i.country = location[0]                    
                i.city = location[1]                    
                print('Update location', ip_address, 'to', location, 'from', loc)
                i.save()     
                time.sleep(random.choice(waiting_list)) # sleep 1 detik agar tidak kentara
        else:
            while datetime.datetime.now() < stop_time:
                print('Waiting for 1 minute')
                time.sleep(random.choice(waiting_list)) # sleep 5 detik

            # reset count
            count = 0
            start_time = datetime.datetime.now()
            stop_time = start_time + timedelta(minutes=1)
            print('Reset variable')

        



import calendar
import datetime
# 
# from datetime import timedelta

def get_last_day_of_month(year, month):
    return calendar.monthrange(year,month)[1]

def add_months(sourcedate, months):
    month = sourcedate.month - 1 + months
    year = sourcedate.year + month // 12
    month = month % 12 + 1
    # day = min(sourcedate.day, calendar.monthrange(year,month)[1])
    day = min(sourcedate.day, get_last_day_of_month(year, month))
    return datetime.datetime(year, month, day)

def get_week_date(year, month, day):
    '''
        Ambil tanggal awal dan akhir 

        misal : calendar (2022,12)
        tgl skrg = 3
        maka ambil tanggal awal = 28/11 
        tanggal akhir = 4/12 

        [0, 0, 0, 1, 2, 3, 4]
        [5, 6, 7, 8, 9, 10, 11]
        [12, 13, 14, 15, 16, 17, 18]
        [19, 20, 21, 22, 23, 24, 25]
        [26, 27, 28, 29, 30, 31, 0]
        
    '''
    cal = calendar.Calendar()
    cal = cal.monthdatescalendar(year, month)
    mfound = False
    i = 0 # init

    for i in range(0, len(cal)-1):
        for j in cal[i]:
            if j.day == day:
                mfound = True
                break

        if mfound: break

    # index ada di i
    week_begin = cal[i][0]
    week_begin = datetime.datetime(week_begin.year, week_begin.month, week_begin.day, 0, 0, 0)
    week_end = cal[i][6]
    week_end = datetime.datetime(week_end.year, week_end.month, week_end.day, 23, 59, 59)

    # untuk last week tinggal dikurangi 7 hari week_begin dan week_end
    return (week_begin, week_end) # return tuple



# Generated by Django 4.1.7 on 2023-03-24 01:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outbox_hitcount', '0002_hitbrowser_version_hitdevice_version_hitos_version'),
    ]

    operations = [
        migrations.AlterField(
            model_name='hitcount',
            name='end_date',
            field=models.DateTimeField(),
        ),
    ]



# Generated by Django 4.1.7 on 2023-03-22 08:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outbox_hitcount', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='hitbrowser',
            name='version',
            field=models.CharField(default=None, max_length=10),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='hitdevice',
            name='version',
            field=models.CharField(default=None, max_length=10),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='hitos',
            name='version',
            field=models.CharField(default=None, max_length=10),
            preserve_default=False,
        ),
    ]
